import os
import yt_dlp
import ytdebunk.settings as settings
# import settings as settings

def download_audio(video_url, start_time=None, end_time=None, verbose=False, ignore_ssl_cert=True):
    if verbose:
        print(f"[ytdebunk-download] Downloading audio from {video_url}...")

    os.makedirs(settings.OUTPUT_DIRECTORY, exist_ok=True)

    if os.path.exists(settings.AUDIO_FILE):
        os.remove(settings.AUDIO_FILE)

    postprocessor_args = []
    if start_time is not None or end_time is not None:
        postprocessor_args = ["-ss", str(start_time)] if start_time is not None else []
        if end_time is not None:
            postprocessor_args += ["-to", str(end_time)]

    ydl_opts = {
        'format': 'bestaudio/best',
        'postprocessors': [{
            'key': 'FFmpegExtractAudio',
            'preferredcodec': 'mp3',
            'preferredquality': '192',
        }],
        'postprocessor_args': postprocessor_args,
        'outtmpl': os.path.splitext(settings.AUDIO_FILE)[0],
        'noplaylist': True,
        'progress_hooks': [lambda d: print(f" Downloading: {d.get('_percent_str', '0%')} complete")],
        'cachedir': False,
        'quiet': not verbose,
        'nocheckcertificate': ignore_ssl_cert,
    }

    with yt_dlp.YoutubeDL(ydl_opts) as ydl:
        ydl.download([video_url])
        if verbose:
            print(f"[ytdebunk-download] Audio download complete! Saved at {settings.AUDIO_FILE}")

if __name__ == "__main__":
    download_audio("https://www.youtube.com/shorts/l8UDM4bOhpU", verbose=True)


import google.generativeai as genai
import ytdebunk.settings as settings
import os

from dotenv import load_dotenv
load_dotenv()

def chunk_text(text, max_chars=3000):
    sentences = text.split("।")
    chunks, current_chunk = [], ""

    for sentence in sentences:
        if len(current_chunk) + len(sentence) < max_chars:
            current_chunk += sentence + "।"
        else:
            chunks.append(current_chunk.strip())
            current_chunk = sentence + "।"

    if current_chunk:  
        chunks.append(current_chunk.strip())

    return chunks


def enhance_transcription(transcription, 
                          key=os.getenv("GENAI_API_KEY"), 
                          verbose=False, 
                          language=settings.LANUAGE_DEFAULT):

    enchacement_prompt = f"""
You are a {settings.TRANSCRIPTION_MODEL_NAMES[language]} language expert. You have been asked to improve the following {settings.TRANSCRIPTION_MODEL_NAMES[language]} transcription by correcting errors and enhancing readability.

IMPORTANT:
1. Return only the transcription without any additional information or instructions.
2. Do not change the meaning of the transcription.
3. Do not add any new information to the transcription.
4. Do not remove any information from the transcription.

Here is the transcription:

"""

    genai.configure(api_key=key)
    model = genai.GenerativeModel('gemini-1.5-flash')
    chunks = chunk_text(transcription)
    refined_chunks = []

    print(f"[ytdebunk-refiner] Refining {len(chunks)} chunks of text...")
    for chunk in chunks:
        if verbose:
            print("[ytdebunk-refiner] Refining chunk no. ", chunks.index(chunk) + 1)
        prompt = enchacement_prompt + chunk
        response = model.generate_content(prompt)
        refined_chunks.append(response.text.strip())
    return " ".join(refined_chunks)


OUTPUT_DIRECTORY = 'output'
AUDIO_FILE = OUTPUT_DIRECTORY + '/audio.mp3'
VIDEO_FILE = OUTPUT_DIRECTORY + '/video.mp4'
TRANSCRIPTION_FILE = OUTPUT_DIRECTORY + '/transcription.txt'
REFINED_TRANSCRIPTION_FILE = OUTPUT_DIRECTORY + '/refined_transcription.txt'
LOGICAL_FAULTS_FILE = OUTPUT_DIRECTORY + '/logical_faults.txt'

LANUAGE_DEFAULT = "en"

TRANSCRIPTION_MODELS = {
    "en": "openai/whisper-small",
    "bn": "bangla-speech-processing/BanglaASR"
}

TRANSCRIPTION_MODEL_NAMES = {
    "en": "English",
    "bn": "Bengali"
}



from setuptools import setup, find_packages

setup(
    name="ytdebunk",
    version="1.1.1",
    packages=find_packages(),
    install_requires=[
        "numpy<2",
        "python-dotenv==1.0.1",
        "google-generativeai==0.8.4",
        "yt-dlp==2025.3.21",
        "torch==2.1.0",
        "torchaudio==2.1.0",
        "librosa==0.11.0",
        "transformers==4.36.2",
    ],
    entry_points={"console_scripts": ["ytdebunk=ytdebunk.ytdebunk:main"]},
    author="Md. Sazzad Hissain Khan",
    author_email='hissain.khan@gmail.com',
    description="A CLI tool to download audio from a YouTube video, transcribe it, and refine and find logical errors in the transcription using AI.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/hissain/youtuber-debunked',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    keywords=[
        "youtube", 
        "transcription", 
        "audio", 
        "refinement", 
        "ai",
        "bengali", 
        "geminai",
        "librosa",
        "streamlit",
        ],
)


import argparse
import os
import ytdebunk.settings as settings
from dotenv import load_dotenv
from ytdebunk.downloader import download_audio
from ytdebunk.transcriber import transcribe_audio
from ytdebunk.refiner import enhance_transcription
from ytdebunk.philosopher import detect_logical_faults

load_dotenv()

def main():
    parser = argparse.ArgumentParser(description="This command aims to automate the extraction of MP3 audio from a given YouTube video link, transcribe the audio content, and analyze the transcriptions using Agentic AI to identify logical fallacies and incorrect claims made by YouTubers.")
    parser.add_argument("yt_video_url", type=str, help="URL of the YouTube video (shorts, video) to be analyzed")
    parser.add_argument("-l", "--language", type=str, default="en", help="Language (code) of the transcription. Valid: [bn, en], default: en")
    parser.add_argument("-e", "--enhance", action="store_true", default=False, help="Enhance the transcription")
    parser.add_argument("-d", "--detect", action="store_true", default=False, help="Detect logical fallacies, bias, irony, faults in the transcription")
    parser.add_argument("-v", "--verbose", action="store_true", default=False, help="Increase output verbosity")
    parser.add_argument("-t", "--token", type=str, help="API token for the Gemini API")
    parser.add_argument("-st", "--start_time", type=float, default=None, help="Start time of the audio clip in seconds for trasncription")
    parser.add_argument("-et", "--end_time", type=float, default=None, help="End time of the audio clip in seconds for trasncription")
    parser.add_argument("-is", "--ignore_ssl", action="store_true", default=True, help="Ignore SSL certificate errors (nocheckcertificate)")
    # parser.add_argument("-debug", "--debug", action="store_true", help="Used for debugging purpose")
    
    args = parser.parse_args()

    if args.enhance:
        token = args.token or os.getenv("GEMINI_API_TOKEN") or os.getenv("GEMINI_API_KEY")
        if not token:
            print("[ytdebunk] Error: Enhancement is enabled but no Gemini API token provided or found in env.")
            return

    st = args.start_time
    et = args.end_time
    ln = args.language

    if ln is not None and ln not in ["bn", "en"]:
        print("[ytdebunk] Error: Invalid language. Valid: [bn, en]")
        return
    
    if st is not None and et is not None and st >= et:
        print("[ytdebunk] Error: Start time must be less than end time.")
        return
    
    download_audio(args.yt_video_url, start_time=st, end_time=et, verbose=args.verbose)

    transcription = transcribe_audio(verbose=args.verbose, 
                                     start_time=st, 
                                     end_time=et,
                                     language = ln)
    
    with open(settings.TRANSCRIPTION_FILE, "w", encoding="utf-8") as f:
            f.write(transcription)
            if args.verbose:
                print(f"[ytdebunk] Transcription saved at {settings.TRANSCRIPTION_FILE}")

    if args.enhance:
        transcription = enhance_transcription(transcription, token, verbose=args.verbose, language=ln)
        with open(settings.REFINED_TRANSCRIPTION_FILE, "w", encoding="utf-8") as f:
            f.write(transcription)
            if args.verbose:
                print(f"[ytdebunk] Refined transcription saved at {settings.REFINED_TRANSCRIPTION_FILE}")

    logical_faults = detect_logical_faults(transcription, verbose=args.verbose, language=ln)
    
    with open(settings.LOGICAL_FAULTS_FILE, "w", encoding="utf-8") as f:
        f.write(logical_faults)
        if args.verbose:
            print(f"[ytdebunk] Logical faults saved at {settings.LOGICAL_FAULTS_FILE}")
    
    if args.verbose:
        print(f"*"*80)
        print(f"[ytdebunk] TRANSCRIPTION:\n{transcription}")
        print(f"-"*80)
        print(f"[ytdebunk] REFINED TRANSCRIPTION:\n{transcription}")
        print(f"-"*80)
        print(f"[ytdebunk] LOGICAL FAULTS:\n{logical_faults}")
        print(f"*"*80)

    return transcription, logical_faults



import librosa
import torch
import torchaudio
import numpy as np
import ytdebunk.settings as settings
from transformers import WhisperTokenizer, WhisperProcessor, WhisperFeatureExtractor, WhisperForConditionalGeneration
import warnings
warnings.filterwarnings("ignore", category=FutureWarning, module="huggingface_hub")

chunk_duration = 15  # Seconds per chunk
sampling_rate_target = 16000

def load_model(model_path, device):
    print("[ytdebunk-transcriber] Loading WhisperFeatureExtractor model...")
    feature_extractor = WhisperFeatureExtractor.from_pretrained(model_path)
    tokenizer = WhisperTokenizer.from_pretrained(model_path)
    processor = WhisperProcessor.from_pretrained(model_path)
    model = WhisperForConditionalGeneration.from_pretrained(model_path).to(device)
    return feature_extractor, tokenizer, processor, model

def transcribe_audio(
        audio_path=settings.AUDIO_FILE, 
        start_time=None, 
        end_time=None,
        verbose=False,
        language=settings.LANUAGE_DEFAULT):
    
    model_path = settings.TRANSCRIPTION_MODELS[language]
    device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
    feature_extractor, tokenizer, processor, model = load_model(model_path, device)
    
    speech_array, sampling_rate = torchaudio.load(audio_path)
    speech_array = speech_array[0].numpy()
    speech_array = librosa.resample(speech_array, orig_sr=sampling_rate, target_sr=sampling_rate_target)
    
    start_sample = int((start_time or 0) * sampling_rate_target)
    end_sample = int(end_time * sampling_rate_target) if end_time is not None else len(speech_array)
    speech_array = speech_array[start_sample:end_sample]
    
    chunk_size = chunk_duration * sampling_rate_target
    num_chunks = int(np.ceil(len(speech_array) / chunk_size))

    transcriptions = []

    print(f"[ytdebunk-transcriber] Transcribing {num_chunks} chunks...")
    for i in range(num_chunks):
        if verbose:
            print(f"[ytdebunk-transcriber] Transcribing chunk {i + 1} of {num_chunks}...")
        start = i * chunk_size
        end = min((i + 1) * chunk_size, len(speech_array))
        
        chunk = speech_array[start:end]
        input_features = feature_extractor(chunk, sampling_rate=sampling_rate_target, return_tensors="pt").input_features.to(device)

        predicted_ids = model.generate(input_features)[0]
        transcription = processor.decode(predicted_ids, skip_special_tokens=True)
        transcriptions.append(transcription)

    full_transcription = " ".join(transcriptions)

    if verbose:
        print(f"[ytdebunk-transcriber] Transcription complete!")

    return full_transcription



import google.generativeai as genai
import os
import ytdebunk.settings as settings
from dotenv import load_dotenv
load_dotenv()

def chunk_text(text, max_chars=3000):
    sentences = text.split("।")
    chunks, current_chunk = [], ""

    for sentence in sentences:
        # Prevent chunks from exceeding max_chars, even if they don't end with "।"
        if len(current_chunk) + len(sentence) + 1 <= max_chars:
            current_chunk += sentence + "।"
        else:
            if current_chunk:
                chunks.append(current_chunk.strip())
            current_chunk = sentence + "।"

    if current_chunk:
        chunks.append(current_chunk.strip())

    return chunks



def detect_logical_faults(transcription, 
                          key=os.getenv("GENAI_API_KEY"), 
                          verbose=False, 
                          language=settings.LANUAGE_DEFAULT):

    fault_detection_prompt = f"""
You are a {settings.TRANSCRIPTION_MODEL_NAMES[language]} language expert and a philosopher specializing in detecting logical flows, fallacies, bias, and irony in a {settings.TRANSCRIPTION_MODEL_NAMES[language]} speaker's content. 
Please be precise and critical while evaluating a piece of {settings.TRANSCRIPTION_MODEL_NAMES[language]} content from a {settings.TRANSCRIPTION_MODEL_NAMES[language]}-speaking YouTuber.

IMPORTANT:
1. Generate in {settings.TRANSCRIPTION_MODEL_NAMES[language]} only.
2. Keep the fallacy, bias, irony, and logical faults in the same order as they appear in the content.
3. Keep the summary concise and to the point and withing single large paragraph without showing point by point.

Here is the speaker's text:
"""

    genai.configure(api_key=key)
    model = genai.GenerativeModel('gemini-1.5-flash')

    chunks = chunk_text(transcription)
    refined_chunks = []

    print(f"[ytdebunk-analyzer] Analyzing {len(chunks)} chunks of text...")
    for idx, chunk in enumerate(chunks):
        if verbose:
            print(f"[ytdebunk-analyzer] Analyzing chunk {idx + 1} of {len(chunks)}")
        prompt = fault_detection_prompt + chunk
        try:
            response = model.generate_content(prompt)
            refined_chunks.append(response.text.strip())
        except Exception as e:
            print(f"[ytdebunk-analyzer] Error while processing chunk {idx + 1}: {e}")
            refined_chunks.append(f"[ytdebunk-analyzer] Error processing chunk {idx + 1}")

    return " ".join(refined_chunks)